@@ -19,18 +19,18 @@
 
 ## wrapper for recluster functions
 
-recluster <- function(macro, dsc, type="auto", ...) UseMethod("recluster")
+recluster <- function(macro, micro, type="auto", ...) UseMethod("recluster")
 
-recluster.DSC <- function(macro, dsc, type="auto", ...) {
+recluster.DSC <- function(macro, micro, type="auto", ...) {
     stop(gettextf("recluster not implemented for class '%s'.", 
 		    paste(class(macro), collapse=", ")))
 }
 
 ### reclustering is done with a DSC_Macro object!
-recluster.DSC_Macro <- function(macro, dsc, type="auto", ...) {
-    cen <- get_centers(dsc, type=type)
+recluster.DSC_Macro <- function(macro, micro, type="auto", ...) {
+    cen <- get_centers(micro, type=type)
     dsd <- DSD_Memory(cen)
-    weight <- get_weights(dsc, scale=NULL, type=type)
+    weight <- get_weights(micro, scale=NULL, type=type)
     update(macro, dsd, n=nrow(cen), weight=weight, ...)
 }
 
